__all__ = ["read_file"]
import subprocess

import rich


from .. import config


def read_file(file_path: str) -> str:
    """Read the contents of a file and return it as a string.

    Helpful when additional source code is required for a particular file,
    outside of what is included in the `git diff`.

    Required when committing untracked files, and their contents don't appear in the `git diff`.

    """
    if not config.Config.quiet:
        rich.print(f"Reading file at [bold magenta]{file_path}[/]")
    with open(file_path, "r") as file:
        return file.read()


def ls(directory: str = ".") -> str:
    """List all contents of a directory in long format, with long times.

    Helpful for seeing when files were last modified.

    """
    if not config.Config.quiet:
        rich.print(f"Running [bold magenta]ls -alhT {directory}[/]")
    return subprocess.check_output(["ls", "-alhT", directory], text=True)



__all__ = ["co_mit"]
import llama_index.core.agent.workflow as llama_agent_workflow
import llama_index.llms.openai
import rich

from . import config, tools


SYSTEM_PROMPT = """
You are an experienced software developer working on a project.
You have made some changes to the codebase and you need to write a commit message to describe the changes you have made.
The most important thing to understand is the changes you have made and why you have made them.
If you need to determine how long this commit took to develop, you must check the commit history and the last modified times of the files in the directory.
"""


def create_user_msg() -> str:
    if not config.Config.example:
        msg = "Create a commit message using conventional commit format."
    else:
        msg = f"Create a commit message using this format:\n'''{config.Config.example}\n'''"
    msg += "\nYou should return only the commit message, without additional information, backticks, quotes, or any other formatting."
    return msg


async def co_mit() -> None:
    commit_agent = llama_agent_workflow.AgentWorkflow.from_tools_or_functions(
        [
            tools.git.diff,
            tools.git.diff_cached,
            tools.git.log,
            tools.git.status,
            tools.os.ls,
            tools.os.read_file,
        ],
        llm=llama_index.llms.openai.OpenAI(
            model="gpt-4o", api_key=config.Config.openai_api_key
        ),
        system_prompt=SYSTEM_PROMPT,
    )

    msg = create_user_msg()
    result = await commit_agent.run(user_msg=msg)
    if not config.Config.quiet:
        rich.print("[bold green]Done.[/]")
    rich.print(str(result))



import rich
import rich_click as click

from . import config


@click.command()
@click.option(
    "--openai-api-key",
    "-k",
    type=click.STRING,
    help="OpenAI API key. Can also set with CO_MIT_OPENAI_API_KEY environment variable.",
)
@click.option(
    "--example",
    "-e",
    type=click.STRING,
    help="Example input to generate a commit message from. Can also set with CO_MIT_EXAMPLE environment variable.",
)
@click.option(
    "--quiet",
    "-q",
    is_flag=True,
    help="Suppress all output other than final commit message. Useful for scripting. Can also set with CO_MIT_QUIET environment variable.",
)
@click.option(
    "--version",
    "-v",
    is_flag=True,
    help="Show version information.",
)
@click.option(
    "--show-config",
    "-c",
    is_flag=True,
    help="Show current configuration.",
)
def main(
    openai_api_key: str | None,
    example: str | None,
    quiet: bool,
    version: bool,
    show_config: bool,
) -> None:
    """Helps with git commits."""

    # Set configuration options
    if openai_api_key:
        config.Config.openai_api_key = openai_api_key
    if example:
        config.Config.example = example
    if quiet:
        config.Config.quiet = quiet

    # Handle alternative commands
    if version:
        from . import __about__

        click.echo(f"co-mit version {__about__.__version__}")
        return
    if show_config:
        import json

        cfg = config.Config.model_dump()
        if cfg["openai_api_key"]:
            cfg["openai_api_key"] = cfg["openai_api_key"][:4] + "*" * 16
        rich.print(json.dumps(cfg, indent=4))
        return

    # Echo before lazy importing to speed up initial message
    if not config.Config.quiet:
        rich.print("[bold yellow]Generating commit message...[/]")

    # Lazy imports to speed up commands that won't run this code
    import asyncio
    from . import commit

    asyncio.run(commit.co_mit())



__all__ = ["Config"]
import pydantic_settings


class _Config(pydantic_settings.BaseSettings, env_prefix="CO_MIT_"):
    example: str | None = None
    openai_api_key: str | None = None
    quiet: bool = False


Config = _Config()



__all__ = ["diff", "diff_cached", "status"]
import subprocess
import rich

from .. import config


def diff() -> str:
    """Return the `git diff` as a string.

    Helpful for seeing all changes not staged for commit.

    """
    if not config.Config.quiet:
        rich.print("Running [bold magenta]git diff[/]")
    return subprocess.check_output(["git", "diff"], text=True)


def diff_cached() -> str:
    """Return the `git diff --cached` as a string.

    Helpful for seeing all changes already staged to be committed.

    """
    if not config.Config.quiet:
        rich.print("Running [bold magenta]git diff --cached[/]")
    return subprocess.check_output(["git", "diff", "--cached"], text=True)


def status() -> str:
    """Return the git status as a string.

    Helpful for seeing a summary of staged and unstaged changes.
    This is usually a good place to start when writing a commit message.

    """
    if not config.Config.quiet:
        rich.print("Running [bold magenta]git status[/]")
    return subprocess.check_output(["git", "status"], text=True)


def log(n: int = 5) -> str:
    """Return the git log as a string. Shows the last `n` commits (up to a maxiumum of 15).

    Helpful for seeing a summary of the commit history, and determining the
    timing of recent commits.

    """
    n = min(n, 15)
    if not config.Config.quiet:
        rich.print(f"Running [bold magenta]git log -n {n}[/]")
    return subprocess.check_output(["git", "log", "-n", str(n)], text=True)
